package cn.teammodel.controller.admin.service.impl;

import cn.teammodel.controller.admin.service.CommonService;
import cn.teammodel.model.dto.admin.common.GCDto;
import cn.teammodel.model.entity.school.ClassInfo;
import cn.teammodel.model.entity.school.School;
import cn.teammodel.model.vo.admin.GradeAndClassVo;
import cn.teammodel.repository.ClassRepository;
import cn.teammodel.repository.SchoolRepository;
import cn.teammodel.utils.MonthToNumberConverter;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.Month;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

import static java.time.LocalDate.now;

@Service
public class CommonServiceImpl implements CommonService {
    @Resource
    private SchoolRepository schoolRepository;
    @Resource
    private ClassRepository classRepository;

    @Override
    public List<GradeAndClassVo> getGradeAndClass(GCDto gcDto) {
        List<GradeAndClassVo> gradeAndClassVos = new ArrayList<>();
        try {
            //获取当前学校该学段下详细信息
            List<School.Period> period = schoolRepository.findPeriodById(gcDto.getSchoolId(), gcDto.getPeriodId());
            List<ClassInfo> classes = classRepository.findClassBySchoolIdAndPeriodId(gcDto.getSchoolId(), gcDto.getPeriodId());
            int year = now().getYear();
            Month month =   now().getMonth();
            int mon =  MonthToNumberConverter.convertMonthToNumber(month.name());
            int day = now().getDayOfMonth();
            //处理年级ID
            for (ClassInfo classInfo : classes) {
                if(period.get(0).getId().equalsIgnoreCase(classInfo.getPeriodId())) {
                    for (School.Semester semester : period.get(0).getSemesters()) {
                        int time = 0;
                        if(semester.getStart() == 1) {
                            if (mon == semester.getMonth())
                            {
                                time = day >= semester.getDay() ? 0 : 1;
                            }
                            else
                            {
                                time = mon > semester.getMonth() ? 0 : 1;
                            }
                            int eYear = year - time;
                            classInfo.setGrade(eYear- classInfo.getYear());
                        }
                    }

                }
            }
            List<String> grades = period.get(0).getGrades();
            int index = 0;
            //根据当前时间去判定各个年级的入学年
            int time = 0;
            if(period.get(0).getId().equalsIgnoreCase(gcDto.getPeriodId())) {
                for (School.Semester semester : period.get(0).getSemesters()) {
                    if(semester.getStart() == 1) {
                        if (mon == semester.getMonth())
                        {
                            time = day >= semester.getDay() ? 0 : 1;
                        }
                        else
                        {
                            time = mon > semester.getMonth() ? 0 : 1;
                        }
                    }
                }

            }
            for (String grade : grades) {
                GradeAndClassVo gradeAndClassVo = new GradeAndClassVo();
                gradeAndClassVo.setGradeId(index);
                gradeAndClassVo.setYear(year - time - index);
                gradeAndClassVo.setGradeName(grade);
                classes.stream().filter(classInfo -> classInfo.getGrade() == gradeAndClassVo.getGradeId()).forEach(classInfo -> {
                    GradeAndClassVo.CI ci = new GradeAndClassVo.CI();
                    ci.setClassId(classInfo.getId());
                    ci.setClassName(classInfo.getName());
                    ci.setYear(classInfo.getYear());
                    gradeAndClassVo.AddClass(ci);
                });
                index ++;
                gradeAndClassVos.add(gradeAndClassVo);
            }

            return gradeAndClassVos;
        }catch (Exception e) {

            throw new RuntimeException("获取当前学校当前学段年级班级信息失败");
        }
    }
}
